﻿/// RESTApiDemo
//
// Copyright © 2018 Eurostep
//
// All rights reserved.
//
// MIT License
//
// Permission is hereby granted, free of charge, to any person obtaining a copy of
// this software and associated documentation files (the "Software"), to deal in
// the Software without restriction, including without limitation the rights to
// use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies of
// the Software, and to permit persons to whom the Software is furnished to do so,
// subject to the following conditions:
//
// The above copyright notice and this permission notice shall be included in all
// copies or substantial portions of the Software.
//
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS
// FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR
// COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER
// IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
// CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
//

using Newtonsoft.Json.Linq;
using System;
using System.Collections.Generic;
using System.Net.Http;
using System.Net.Http.Headers;
using System.Text;
using System.Threading.Tasks;

namespace RESTApiDemo
{
    public class BasicAuthenticationHeaderValue : AuthenticationHeaderValue
    {
        public BasicAuthenticationHeaderValue(string clientId, string clientSecret)
            : base("Basic", EncodeCredential(clientId, clientSecret))
        {
        }

        private static string EncodeCredential(string clientId, string clientSecret)
        {
            return Convert.ToBase64String(Encoding.UTF8.GetBytes(string.Format("{0}:{1}", new object[2]
            {
                clientId,
                clientSecret
            })));
        }
    }

    internal class Program
    {
        private static void Main(string[] args)
        {
            Run().Wait();
        }

        private static string Base64UrlEncode(byte[] input)
        {
            var output = Convert.ToBase64String(input);
            output = output.Split('=')[0]; // Remove any trailing '='s
            output = output.Replace('+', '-'); // 62nd char of encoding
            output = output.Replace('/', '_'); // 63rd char of encoding
            return output;
        }

        private static async Task Run()
        {
            // oAuth 2.0 Resource Owner
            System.Net.ServicePointManager.Expect100Continue = false;
            var httpClient = new HttpClient(new HttpClientHandler());
            httpClient.BaseAddress = new Uri("https://server.machine.net/AuthorizationServer/sasweb/oauth/token");
            httpClient.DefaultRequestHeaders.Authorization = new BasicAuthenticationHeaderValue("NovaWebApi", "CLIENTSECRET");

            var form = new Dictionary<string, string>
            {
                {"grant_type", "password"},
                {"username", "{email}"},
                {"password", "{password}"},
                {"scope", "All" }
            };

            HttpResponseMessage message = await httpClient.PostAsync(string.Empty, new FormUrlEncodedContent(form));
            message.EnsureSuccessStatusCode();
            string content = await message.Content.ReadAsStringAsync();

            var token = JObject.Parse(content);

            // Refresh token
            form = new Dictionary<string, string>
            {
                {"grant_type", "refresh_token"},
                {"refresh_token", token["refresh_token"].Value<string>()}
            };

            message = await httpClient.PostAsync(string.Empty, new FormUrlEncodedContent(form));
            message.EnsureSuccessStatusCode();
            content = await message.Content.ReadAsStringAsync();

            token = JObject.Parse(content);

            // Http Client on base Uri
            httpClient = new HttpClient();
            httpClient.DefaultRequestHeaders.Authorization = new AuthenticationHeaderValue("Bearer", token["access_token"].Value<string>());
            httpClient.BaseAddress = new Uri("https://server.machine.net/api");

            // Get information filter for space
            var infoFilterResponse = await httpClient.GetAsync("https://server.machine.net/api/space/incontrol/informationFilter");
            infoFilterResponse.EnsureSuccessStatusCode();

            var resultJson = JObject.Parse(await infoFilterResponse.Content.ReadAsStringAsync());

            // Set information filter header
            var dataString = resultJson["data"].ToString();

            var informationFilter = Base64UrlEncode(Encoding.UTF8.GetBytes(dataString));
            httpClient.DefaultRequestHeaders.Add("SAs-InformationFilter", informationFilter);

            var person = new JObject();
            person["id"] = Guid.NewGuid().ToString();
            person["firstName"] = "First";
            person["lastName"] = "Last";

            var jsonContent = new StringContent(person.ToString(), Encoding.UTF8, "application/json");
            var postResponse = await httpClient.PostAsync("https://server.machine.net/api/space/incontrol/softtype/Person", jsonContent);
            postResponse.EnsureSuccessStatusCode();

            var getResponse = await httpClient.GetAsync(postResponse.Headers.Location);

            string getContent = await getResponse.Content.ReadAsStringAsync();

            var personGet = JObject.Parse(getContent);

            var queryResponse = await httpClient.GetAsync("https://server.machine.net/api/space/incontrol/search/COMMON.Part.Design.All?page=1");

            string queryContent = await queryResponse.Content.ReadAsStringAsync();
            var queryResult = JObject.Parse(queryContent);
        }
    }
}